require "thor"
require "rest-client"
require "cdn_manager_pcache_probe/version"
require "cdn_manager_pcache_probe/connector"

module CdnManagerPcacheProbe
  class Cli < Thor
      package_name "cdn_manager_pcache_probe"
      include Thor::Actions

      # catch control-c and exit
      trap("SIGINT") {
        puts " bye"
        exit!
      }

      # exit with return code 1 in case of an error
      def self.exit_on_failure?
        true
      end

      desc "version", "print cdn_manager_pcache_probe version number"
      def version
        say "cdn_manager_pcache_probe v#{VERSION}"
      end
      map %w(-v --version) => :version

      desc "collect", "probe pcache elasticsearch"
      option :url,
        desc: "ElasticSearch URL",
        required: true
      option :key,
        desc: "Limit query on a specific voluem_key"
      option :interval,
        desc: "probe inteval in minutes",
        default: 3,
        type: :numeric
      option :delay,
        desc: "data delay - give elasticsearch some time to receive the logs",
        default: 5,
        type: :numeric
      option :quiet,
        desc: "print minimal output, just Mbit/s",
        type: :boolean
      option :debug,
        desc: "print debug output",
        type: :boolean
      option :api_base_url,
        desc: "cdn-manager-api base url to HTTP-POST result to"
      option :api_key,
        desc: "cdn-manager-api key"
      option :service_name,
        desc: "cdn-manager service name"
      option :metric_name,
        desc: "cdn-manager service name",
        default: "pcache_total_bw"
      def collect
        bytes_sent = CdnManagerPcacheProbe::Connector.new(
          options[:url]
        ).get_probe(options)
        usage = (bytes_sent / (options[:interval] * 60) * 8 / 1024**2).round(3)

        # post result to the cdn-manager-api
        if options[:api_base_url]
          RestClient.log = STDOUT if options[:debug]
          begin
            RestClient.post(
              File.join(
                options[:api_base_url], "api/v1",
                "services", options[:service_name],
                "metrics", options[:metric_name],
                usage.round(0).to_s
              ),
              { key: options[:api_key] }
            )
          rescue => e
            say "Error posting metric: #{e.message}", :red
          end
        end

        if options[:quiet]
          puts usage.round(0)
        else
          if options[:key]
            say "Usage for #{options[:key]} ", :yellow
          else
            say "Overall usage ", :yellow
          end
          say "over #{options[:interval]} min (#{options[:delay]} min delay):", :yellow
          puts "Total volume sent: #{(bytes_sent / 1024**3).round(3)} GB"
          print "Average bandwidth: #{usage} Mbit/s"
          puts usage >= 1000 ? " (#{(usage / 1024.0).round(2)} Gbit/s)" : nil
        end
      end



    end # class

end # module cli
